## SOC500
## Section 1 - Running simulations 
## Simone Zhang, 09/12/2016
## Demonstrates how to run simulations using loops and replicate

## Generate bag (Create vector representing your sample space)
?rep
rep("R",3)
bag <- c(rep("R", 3), rep("B", 2))
bag

## Draw two marbles from the bag
?sample

draw <- sample(bag, 2)
draw

## Check if the two marbles we drew are the same colour
if (draw[1] == draw[2]) {
  same_color <- TRUE
} else {
  same_color <- FALSE
}

same_color

## You can also do the same thing with the ifelse function
same_color <- ifelse(draw[1] == draw[2], TRUE, FALSE)

same_color

## But this only runs one simulation. How do I do this many times? 
## APPROACH 1 - LOOPS 
num_sims <- 10000

sim_results <- rep(NA, num_sims) # Initialize the vector sim_results with NAs  
sim_results

for (i in 1:num_sims) {
  draw <- sample(bag, 2)
  # Check if the two marbles we drew are the same colour
  same_color <- ifelse(draw[1] == draw[2], TRUE, FALSE)
  sim_results[i] <- same_color # Store results in the vector sim_results
}

sim_results
summary(sim_results)
mean(sim_results)

## APPROACH 2 - USE REPLICATE
## First, we need to declare a function
run_simulation <- function(sample_space) {
  draw <- sample(sample_space, 2)
  same_color <- ifelse(draw[1] == draw[2], TRUE, FALSE)
  return(same_color)
}

run_simulation(bag) # Function call

## Second, we use replicate, which is a wrapper for apply to call 
## the function multiple times
?replicate

sim_results_repl <- replicate(num_sims, run_simulation(bag))
sim_results_repl

mean(sim_results_repl)
